#!/usr/bin/env python3

# build-mimeapps.py
# Copyright (C) 2019, Jan Alexander Steffens (heftig) <jan.steffens@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from argparse import ArgumentParser
from configparser import ConfigParser, NoOptionError
from pathlib import Path
from sys import stderr

aparse = ArgumentParser(
    description="""
Transform default associations in custom format into standard XDG format.
"""
)
aparse.add_argument(
    "input", metavar="INPUT", type=Path, help="Custom format file to parse"
)
aparse.add_argument(
    "output", metavar="OUTPUT", type=Path, help="XDG format file to produce"
)
args = aparse.parse_args()

input_path: Path = args.input
output_path: Path = args.output
types: dict[str, list[str]] = {}
section: str = ""


def append_app(mime_type: str, app: str):
    apps = types.setdefault(mime_type, [])
    if app not in apps:
        apps.append(app)


def remove_app(mime_type: str, app: str):
    apps = types.setdefault(mime_type, [])
    try:
        apps.remove(app)
    except ValueError:
        print(f"{mime_type} was not associated with {app}", file=stderr)


with input_path.open() as input_file:
    for line in input_file:
        # Handle whitespace and comments
        line = line.strip()
        if not line or line.startswith("#"):
            continue

        # Section headers
        if line.startswith("[") and line.endswith("]"):
            section = line[1:-1]
            continue

        # Load desktop file
        if section == "Desktop Entry Files":
            path = input_path.parent / line
            if not path.exists():
                path = Path("/usr/share/applications") / line
                if not path.exists():
                    print(f"Couldn't find {line}", file=stderr)
                    continue

            cparse = ConfigParser(
                delimiters=("=",), comment_prefixes=("#",), interpolation=None
            )
            cparse.optionxform = str
            cparse.read(path)

            try:
                mime_types = cparse.get("Desktop Entry", "MimeType")
            except NoOptionError as e:
                print(f"No MimeType in {path}", file=stderr)
                continue

            for mime_type in mime_types.split(";"):
                if not mime_type:
                    continue
                append_app(mime_type, path.name)
            continue

        # Explicit associations
        if section.endswith(".desktop"):
            if line.startswith("-"):
                remove_app(line[1:], section)
            else:
                append_app(line, section)
            continue

        raise RuntimeError(f"Bad section for line {line!r}")

with output_path.open("w") as output_file:
    print("[Default Applications]", file=output_file)

    for mime_type, apps in sorted(types.items()):
        if not apps:
            print("No associations left for", mime_type, file=stderr)
            continue

        if len(apps) > 1:
            print("Priority for", mime_type, end=": ", file=stderr)
            print(*apps, sep=" > ", file=stderr)

        print(mime_type, end="=", file=output_file)
        print(*apps, sep=";", end=";\n", file=output_file)

# Formatted using black
# vim:set et sw=4 tw=88:
